Rename all arch primitives with prefix caa_
[urcu.git] / urcu.c
diff --git a/urcu.c b/urcu.c
index 5e8c6127a8c2f6413d56e95f66eba8c827595c85..b4a5a7db948b8d57a03960747c5257d9631b1c7c 100644 (file)
--- a/urcu.c
+++ b/urcu.c
@@ -3,7 +3,7 @@
  *
  * Userspace RCU library
  *
- * Copyright (c) 2009 Mathieu Desnoyers <mathieu.desnoyers@polymtl.ca>
+ * Copyright (c) 2009 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
  * Copyright (c) 2009 Paul E. McKenney, IBM Corporation.
  *
  * This library is free software; you can redistribute it and/or
@@ -67,7 +67,7 @@ int gp_futex;
  * Also has a RCU_GP_COUNT of 1, to accelerate the reader fast path.
  * Written to only by writer with mutex taken. Read by both writer and readers.
  */
-long rcu_gp_ctr = RCU_GP_COUNT;
+unsigned long rcu_gp_ctr = RCU_GP_COUNT;
 
 /*
  * Written to only by each individual reader. Read by both the reader and the
@@ -99,10 +99,10 @@ static void mutex_lock(pthread_mutex_t *mutex)
                        perror("Error in pthread mutex lock");
                        exit(-1);
                }
-               if (rcu_reader.need_mb) {
-                       smp_mb();
-                       rcu_reader.need_mb = 0;
-                       smp_mb();
+               if (CAA_LOAD_SHARED(rcu_reader.need_mb)) {
+                       cmm_smp_mb();
+                       _CAA_STORE_SHARED(rcu_reader.need_mb, 0);
+                       cmm_smp_mb();
                }
                poll(NULL,0,10);
        }
@@ -126,14 +126,14 @@ static void smp_mb_master(int group)
        if (likely(has_sys_membarrier))
                membarrier(MEMBARRIER_EXPEDITED);
        else
-               smp_mb();
+               cmm_smp_mb();
 }
 #endif
 
 #ifdef RCU_MB
 static void smp_mb_master(int group)
 {
-       smp_mb();
+       cmm_smp_mb();
 }
 #endif
 
@@ -143,20 +143,19 @@ static void force_mb_all_readers(void)
        struct rcu_reader *index;
 
        /*
* Ask for each threads to execute a smp_mb() so we can consider the
         * compiler barriers around rcu read lock as real memory barriers.
         */
        if (list_empty(&registry))
                return;
        /*
* pthread_kill has a smp_mb(). But beware, we assume it performs
         * a cache flush on architectures with non-coherent cache. Let's play
* safe and don't assume anything : we use smp_mc() to make sure the
         * cache flush is enforced.
         */
-       list_for_each_entry(index, &registry, head) {
-               index->need_mb = 1;
-               smp_mc();       /* write need_mb before sending the signal */
+       list_for_each_entry(index, &registry, node) {
+               CAA_STORE_SHARED(index->need_mb, 1);
                pthread_kill(index->tid, SIGRCU);
        }
        /*
@@ -172,13 +171,13 @@ static void force_mb_all_readers(void)
         * relevant bug report.  For Linux kernels, we recommend getting
         * the Linux Test Project (LTP).
         */
-       list_for_each_entry(index, &registry, head) {
-               while (index->need_mb) {
+       list_for_each_entry(index, &registry, node) {
+               while (CAA_LOAD_SHARED(index->need_mb)) {
                        pthread_kill(index->tid, SIGRCU);
                        poll(NULL, 0, 1);
                }
        }
-       smp_mb();       /* read ->need_mb before ending the barrier */
+       cmm_smp_mb();   /* read ->need_mb before ending the barrier */
 }
 
 static void smp_mb_master(int group)
@@ -206,21 +205,23 @@ void update_counter_and_wait(void)
        struct rcu_reader *index, *tmp;
 
        /* Switch parity: 0 -> 1, 1 -> 0 */
-       STORE_SHARED(rcu_gp_ctr, rcu_gp_ctr ^ RCU_GP_CTR_PHASE);
+       CAA_STORE_SHARED(rcu_gp_ctr, rcu_gp_ctr ^ RCU_GP_CTR_PHASE);
 
        /*
-        * Must commit qparity update to memory before waiting for other parity
-        * quiescent state. Failure to do so could result in the writer waiting
-        * forever while new readers are always accessing data (no progress).
-        * Ensured by STORE_SHARED and LOAD_SHARED.
+        * Must commit rcu_gp_ctr update to memory before waiting for quiescent
+        * state. Failure to do so could result in the writer waiting forever
+        * while new readers are always accessing data (no progress). Enforce
+        * compiler-order of store to rcu_gp_ctr before load rcu_reader ctr.
         */
+       cmm_barrier();
 
        /*
* Adding a smp_mb() which is _not_ formally required, but makes the
         * model easier to understand. It does not have a big performance impact
         * anyway, given this is the write-side.
         */
-       smp_mb();
+       cmm_smp_mb();
 
        /*
         * Wait for each thread rcu_reader.ctr count to become 0.
@@ -233,9 +234,9 @@ void update_counter_and_wait(void)
                        smp_mb_master(RCU_MB_GROUP);
                }
 
-               list_for_each_entry_safe(index, tmp, &registry, head) {
-                       if (!rcu_old_gp_ongoing(&index->ctr))
-                               list_move(&index->head, &qsreaders);
+               list_for_each_entry_safe(index, tmp, &registry, node) {
+                       if (!rcu_gp_ongoing(&index->ctr))
+                               list_move(&index->node, &qsreaders);
                }
 
 #ifndef HAS_INCOHERENT_CACHES
@@ -250,7 +251,7 @@ void update_counter_and_wait(void)
                        if (wait_loops == RCU_QS_ACTIVE_ATTEMPTS)
                                wait_gp();
                        else
-                               cpu_relax();
+                               caa_cpu_relax();
                }
 #else /* #ifndef HAS_INCOHERENT_CACHES */
                /*
@@ -274,7 +275,7 @@ void update_counter_and_wait(void)
                                wait_loops = 0;
                                break; /* only escape switch */
                        default:
-                               cpu_relax();
+                               caa_cpu_relax();
                        }
                }
 #endif /* #else #ifndef HAS_INCOHERENT_CACHES */
@@ -303,18 +304,19 @@ void synchronize_rcu(void)
 
        /*
         * Must finish waiting for quiescent state for parity 0 before
-        * committing qparity update to memory. Failure to do so could result in
-        * the writer waiting forever while new readers are always accessing
-        * data (no progress).
-        * Ensured by STORE_SHARED and LOAD_SHARED.
+        * committing next rcu_gp_ctr update to memory. Failure to do so could
+        * result in the writer waiting forever while new readers are always
+        * accessing data (no progress).  Enforce compiler-order of load
+        * rcu_reader ctr before store to rcu_gp_ctr.
         */
+       cmm_barrier();
 
        /*
* Adding a smp_mb() which is _not_ formally required, but makes the
         * model easier to understand. It does not have a big performance impact
         * anyway, given this is the write-side.
         */
-       smp_mb();
+       cmm_smp_mb();
 
        /*
         * Wait for previous parity to be empty of readers.
@@ -347,18 +349,18 @@ void rcu_register_thread(void)
 {
        rcu_reader.tid = pthread_self();
        assert(rcu_reader.need_mb == 0);
-       assert(rcu_reader.ctr == 0);
+       assert(!(rcu_reader.ctr & RCU_GP_CTR_NEST_MASK));
 
        mutex_lock(&rcu_gp_lock);
        rcu_init();     /* In case gcc does not support constructor attribute */
-       list_add(&rcu_reader.head, &registry);
+       list_add(&rcu_reader.node, &registry);
        mutex_unlock(&rcu_gp_lock);
 }
 
 void rcu_unregister_thread(void)
 {
        mutex_lock(&rcu_gp_lock);
-       list_del(&rcu_reader.head);
+       list_del(&rcu_reader.node);
        mutex_unlock(&rcu_gp_lock);
 }
 
@@ -377,13 +379,13 @@ void rcu_init(void)
 static void sigrcu_handler(int signo, siginfo_t *siginfo, void *context)
 {
        /*
* Executing this smp_mb() is the only purpose of this signal handler.
* It punctually promotes barrier() into smp_mb() on every thread it is
         * executed on.
         */
-       smp_mb();
-       rcu_reader.need_mb = 0;
-       smp_mb();
+       cmm_smp_mb();
+       _CAA_STORE_SHARED(rcu_reader.need_mb, 0);
+       cmm_smp_mb();
 }
 
 /*
@@ -143,20 +143,19 @@ static void force_mb_all_readers(void)
 
        /*
-        * Ask for each threads to execute a smp_mb() so we can consider the
+        * Ask for each threads to execute a cmm_smp_mb() so we can consider the
         * compiler barriers around rcu read lock as real memory barriers.
         */
@@ -143,20 +143,19 @@ static void force_mb_all_readers(void)
                return;
        /*
-        * pthread_kill has a smp_mb(). But beware, we assume it performs
+        * pthread_kill has a cmm_smp_mb(). But beware, we assume it performs
         * a cache flush on architectures with non-coherent cache. Let's play
@@ -143,20 +143,19 @@ static void force_mb_all_readers(void)
         * a cache flush on architectures with non-coherent cache. Let's play
-        * safe and don't assume anything : we use smp_mc() to make sure the
+        * safe and don't assume anything : we use cmm_smp_mc() to make sure the
         * cache flush is enforced.
         */
@@ -206,21 +205,23 @@ void update_counter_and_wait(void)
 
        /*
-        * Adding a smp_mb() which is _not_ formally required, but makes the
+        *
+        * Adding a cmm_smp_mb() which is _not_ formally required, but makes the
         * model easier to understand. It does not have a big performance impact
         * anyway, given this is the write-side.
@@ -303,18 +304,19 @@ void synchronize_rcu(void)
 
        /*
-        * Adding a smp_mb() which is _not_ formally required, but makes the
+        * Adding a cmm_smp_mb() which is _not_ formally required, but makes the
         * model easier to understand. It does not have a big performance impact
         * anyway, given this is the write-side.
@@ -377,13 +379,13 @@ void rcu_init(void)
 {
        /*
-        * Executing this smp_mb() is the only purpose of this signal handler.
-        * It punctually promotes barrier() into smp_mb() on every thread it is
+        * Executing this cmm_smp_mb() is the only purpose of this signal handler.
+        * It punctually promotes cmm_barrier() into cmm_smp_mb() on every thread it is
         * executed on.
         */
This page took 0.026388 seconds and 4 git commands to generate.